# Generated by Django 2.0 on 2018-07-09 10:19

import arkindex.documents.models
from django.db import migrations, models
import django.db.models.deletion
import enumfields.fields
import uuid


POPULATE_TRANSCRIPTIONS = '''
insert into documents_transcriptionnew
(id, type, zone_id, score, text, element_id)
select e.id, e.type, e.zone_id, t.score, t.text, parent.id as element_id
from documents_element as e
inner join documents_transcription as t on (t.element_ptr_id = e.id)
inner join documents_elementpath as ep on (e.id = ep.element_id)
inner join documents_element as parent on (parent.id = ep.path[array_length(ep.path, 1)])
'''

REMOVE_PATHS = '''
delete from documents_elementpath
where element_id in (
    select id from documents_element
    where type in ('paragraph', 'line', 'word', 'character')
)
'''

REMOVE_ELEMENTS = '''
delete from documents_element
where type in ('paragraph', 'line', 'word', 'character')
'''


class Migration(migrations.Migration):

    dependencies = [
        ('images', '0005_zone_polygon_tmp'),
        ('documents', '0020_metadata_revision'),
    ]

    operations = [
        # Create new Transcription model
        migrations.CreateModel(
            name='TranscriptionNew',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', enumfields.fields.EnumField(
                    db_index=True,
                    default='word',
                    enum=arkindex.documents.models.TranscriptionType,
                    max_length=50,
                )),
                ('text', models.TextField(blank=True, null=True)),
                ('score', models.FloatField(blank=True, null=True)),
                ('element', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='transcriptions',
                    to='documents.Element',
                )),
                ('zone', models.ForeignKey(
                    blank=True,
                    null=True,
                    on_delete=django.db.models.deletion.SET_NULL,
                    related_name='transcriptions',
                    to='images.Zone',
                )),
            ],
        ),

        # Populate it with raw sql data
        migrations.RunSQL(POPULATE_TRANSCRIPTIONS, ''),

        # Remove old data
        migrations.DeleteModel(
            name='Transcription',
        ),
        migrations.RunSQL(REMOVE_PATHS, ''),
        migrations.RunSQL(REMOVE_ELEMENTS, ''),

        # Replace original model
        migrations.RenameModel(
            old_name='TranscriptionNew',
            new_name='Transcription',
        ),
    ]
