from rest_framework import serializers
from arkindex.documents.models import Transcription, TranscriptionType, Act
from arkindex.documents.serializers.elements import ElementLightSerializer
from arkindex.documents.serializers.transcriptions import TranscriptionSerializer
from arkindex.images.serializers import ZoneSerializer
from arkindex.project.serializer_fields import EnumField, ViewerURLField


class TranscriptionSearchResultSerializer(serializers.ModelSerializer):
    """
    Link between objects & their search indexation
    """
    type = EnumField(TranscriptionType)
    zone = ZoneSerializer()
    parents = serializers.ListField(
        child=serializers.ListField(
            child=ElementLightSerializer()
        ),
        source='parent_paths',
    )

    class Meta:
        model = Transcription
        fields = (
            'id',
            'type',
            'text',
            'score',
            'zone',
            'parents',
        )


class ActSearchResultSerializer(serializers.ModelSerializer):
    """
    Serialize an act
    """
    transcriptions = TranscriptionSerializer(many=True, source='transcriptions_results')
    surfaces = ZoneSerializer(many=True)
    parents = serializers.ListField(
        child=serializers.ListField(
            child=ElementLightSerializer()
        ),
        read_only=True,
    )
    viewer_url = ViewerURLField('api:act-manifest')

    class Meta:
        model = Act
        fields = (
            'id',
            'name',
            'number',
            'transcriptions',
            'surfaces',
            'parents',
            'viewer_url',
        )
