from rest_framework import serializers
from arkindex.documents.models import Element, Transcription, TranscriptionType
from arkindex.images.models import Image
from arkindex.project.serializer_fields import EnumField


class TranscriptionSerializer(serializers.ModelSerializer):
    """
    Serialises a Transcription
    issued from a search
    """

    type = EnumField(TranscriptionType)

    class Meta:
        model = Transcription
        fields = (
            'id',
            'type',
            'text',
            'score',
        )


class TranscriptionCreateSerializer(serializers.Serializer):
    """
    Allows for insertion of new transcriptions and zones
    """
    element = serializers.PrimaryKeyRelatedField(queryset=Element.objects.all())
    polygon = serializers.ListField(
        child=serializers.ListField(
            child=serializers.IntegerField(),
            min_length=2,
            max_length=2
        ),
        min_length=3
    )
    text = serializers.CharField()
    score = serializers.FloatField(min_value=0, max_value=1)
    type = EnumField(TranscriptionType)


class TranscriptionBulkSerializer(serializers.Serializer):
    """
    Allows for insertion of new transcriptions and zones
    in Bulk (used by serializer below)
    Note: no element !
    """
    polygon = serializers.ListField(
        child=serializers.ListField(
            child=serializers.IntegerField(),
            min_length=2,
            max_length=2
        ),
        min_length=3
    )
    text = serializers.CharField()
    score = serializers.FloatField(min_value=0, max_value=1)
    type = EnumField(TranscriptionType)


class TranscriptionsSerializer(serializers.Serializer):
    """
    Allows for insertion of new transcriptions and zones
    in Bulk (uses serializer above) on a common parent
    """
    transcriptions = TranscriptionBulkSerializer(many=True)
    parent = serializers.PrimaryKeyRelatedField(queryset=Element.objects.all())
    image = serializers.PrimaryKeyRelatedField(queryset=Image.objects.all())
