from django.conf import settings
from django.template.loader import render_to_string
from elasticsearch import Elasticsearch
from rest_framework.exceptions import APIException
import json


class ESQuerySet(object):

    def __init__(
            self, es_index, es_type, post_process, post_process_args=[],
            _source=True, query={}, aggs={}, sort={}):
        assert isinstance(es_index, str)
        assert isinstance(es_type, str)
        assert callable(post_process)
        self.es_index = es_index
        self.es_type = es_type
        self.post_process = post_process
        self.post_process_args = post_process_args
        self._source = _source
        self.query = query
        self.aggs = aggs
        self.sort = sort
        self.elastic = Elasticsearch(settings.ELASTIC_SEARCH_HOSTS)

    def __getitem__(self, value):
        if isinstance(value, int):
            return self[value:value+1]
        if not isinstance(value, slice):
            raise NotImplementedError
        if value.step is not None and value.step != 1:
            raise NotImplementedError
        results = self.elastic.search(
            index=self.es_index,
            body={
                "from": value.start,
                "size": value.stop - value.start,
                "_source": self._source,
                "query": self.query,
                "aggs": self.aggs,
                "sort": self.sort,
            },
            doc_type=self.es_type,
        )
        return self.post_process(results, *self.post_process_args) or []

    def __len__(self):
        return self.count()

    def count(self):
        return self.elastic.count(
            index=self.es_index,
            body={
                "query": self.query,
            },
            doc_type=self.es_type,
        )['count']

    def __iter__(self):
        return self[0:len(self)]

    @staticmethod
    def make_query(template_name, ctx={}):
        """
        Use a Django template to generate a query.
        """
        try:
            return json.loads(render_to_string(template_name, context=ctx))
        except json.JSONDecodeError as e:
            raise APIException('Failed to parse query string')
