"""
Django settings for arkindex project.

Generated by 'django-admin startproject' using Django 1.11.6.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import os
import logging

# Admins in charge
ADMINS = [
    ('Bastien', 'abadie@teklia.com'),
    ('Erwan', 'rouchet@teklia.com'),
]

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
LOGS_DIR = os.environ.get('LOGS_DIR', os.path.realpath(os.path.join(BASE_DIR, '../logs')))
if not os.path.isdir(LOGS_DIR):
    os.makedirs(LOGS_DIR)

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('SECRET_KEY', 'jf0w^y&ml(caax8f&a1mub)(js9(l5mhbbhosz3gi+m01ex+lo')

# SECURITY WARNING: don't run with debug turned on in production!
ARKINDEX_ENV = os.environ.get('ARKINDEX_ENV', 'dev')
DEBUG = ARKINDEX_ENV == 'dev'

hosts = os.environ.get('ALLOWED_HOSTS')
ALLOWED_HOSTS = hosts and hosts.split(',') or []

# Required for django-debug-toolbar
INTERNAL_IPS = ['127.0.0.1', '127.0.1.1']

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # Tools
    'rest_framework',
    'rest_framework.authtoken',
    'webpack_loader',

    # Our apps
    'arkindex.images',
    'arkindex.documents',
    'arkindex.users',
    'arkindex.dataimport',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'arkindex.project.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'arkindex.project.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases
# Use database configured by docker-compose

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': os.environ.get('DB_NAME', 'arkindex_dev'),
        'USER': os.environ.get('DB_USER', 'devuser'),
        'PASSWORD': os.environ.get('DB_PASSWORD', 'devdata'),
        'HOST': os.environ.get('DB_HOST', 'localhost'),
        'PORT': os.environ.get('DB_PORT', 9100),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

AUTH_USER_MODEL = 'users.User'

LOGIN_REDIRECT_URL = 'volumes'
LOGOUT_REDIRECT_URL = 'volumes'
LOGIN_URL = 'login'

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Cookies
CSRF_COOKIE_NAME = 'arkindex.csrf'
SESSION_COOKIE_NAME = 'arkindex.auth'


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/
FRONTEND_DIR = os.environ.get(
    'FRONTEND_DIR', os.path.join(BASE_DIR, '../../frontend/build'))

STATIC_URL = '/static/'
STATIC_ROOT = os.environ.get('STATIC_ROOT')
STATICFILES_DIRS = [
    FRONTEND_DIR,
]

# File uploads
MEDIA_URL = '/media/'
MEDIA_ROOT = os.environ.get('MEDIA_ROOT', os.path.join(BASE_DIR, 'media'))
if not os.path.isdir(MEDIA_ROOT):
    os.makedirs(MEDIA_ROOT)
LOCAL_MEDIA_ROOT = os.environ.get('LOCAL_MEDIA_ROOT', os.path.join(BASE_DIR, 'iiif-users'))
LOCAL_IMAGESERVER_ID = os.environ.get('LOCAL_IMAGESERVER_ID', 1)

# Frontend
WEBPACK_LOADER = {
    'DEFAULT': {
        'BUNDLE_DIR_NAME': '',
        'STATS_FILE': os.path.join(FRONTEND_DIR, 'webpack-stats.json'),
    }
}


# API
REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        # 'rest_framework.permissions.IsAuthenticated',
        # Open bar for demos !
        'rest_framework.permissions.AllowAny',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.TokenAuthentication',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 20,
}

# Elastic search config
ELASTIC_SEARCH_HOSTS = [
    os.environ.get('ES_HOST', 'localhost'),
]
ES_INDEX_TRANSCRIPTIONS = 'transcriptions'
ES_INDEX_ACTS = 'acts'

# Silent logger for elasticsearch
logging.getLogger('elasticsearch').setLevel(logging.WARNING)

# SSLify proxy
SSLIFY_HOST = os.environ.get('SSLIFY_HOST')

# Mirador host setup
MIRADOR_URI = os.environ.get('MIRADOR_URI', 'https://mirador.ark')

# Use SSL proxy
SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')

# IIIF manifests
IIIF_PRESENTATION_CONTEXT = "http://iiif.io/api/presentation/2/context.json"
IIIF_IMAGE_CONTEXT = "http://iiif.io/api/image/2/context.json"
IIIF_SEARCH_CONTEXT = "http://iiif.io/api/search/0/context.json"
IIIF_IMAGE_SERVICE_PROFILE = "http://iiif.io/api/image/2/level2.json"
IIIF_SEARCH_SERVICE_PROFILE = "http://iiif.io/api/search/0/search"

# Set to True to show transcription annotation lists instead of act annotation lists
IIIF_TRANSCRIPTION_LIST = False

# TEI XSLT file path
TEI_XSLT_PATH = os.path.join(BASE_DIR, 'documents/teitohtml.xsl')

# GitLab OAuth
GITLAB_APP_ID = os.environ.get('GITLAB_APP_ID')
GITLAB_APP_SECRET = os.environ.get('GITLAB_APP_SECRET')

# Cache into memcached
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        'LOCATION': '{}:11211'.format(os.environ.get('CACHE_HOST', 'localhost')),
    }
}

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'importers': {
            'level': 'INFO',
            'class': 'logging.FileHandler',
            'formatter': 'verbose',
            'filename': os.path.join(LOGS_DIR, 'importers.log'),
        },
        'cache': {
            'level': 'INFO',
            'class': 'logging.FileHandler',
            'formatter': 'verbose',
            'filename': os.path.join(LOGS_DIR, 'cache.log'),
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler',
            'include_html': True,
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', 'mail_admins', ],
            'level': 'INFO',
            'propagate': True,
        },
        'arkindex.documents.cache': {
            'handlers': ['console', 'cache'],
            'level': 'DEBUG',
        },
        'arkindex.documents.importer': {
            'handlers': ['importers'],
            'level': 'DEBUG',
        },
        'arkindex.documents.surface': {
            'handlers': ['importers'],
            'level': 'DEBUG',
        },
    },
    'formatters': {
        'verbose': {
            'format': '[%(levelname)s] %(asctime)s %(module)s: %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
}

# Async Workers
CELERY_BROKER_URL = os.environ.get('QUEUE_URL', 'redis://localhost:6379/0')
CELERY_RESULT_BACKEND = os.environ.get('QUEUE_URL', 'redis://localhost:6379/0')
CELERY_TRACK_STARTED = True
CELERY_TASK_SERIALIZER = 'pickle'
CELERY_RESULT_SERIALIZER = 'pickle'
CELERY_ACCEPT_CONTENT = ['json', 'pickle']
CELERY_ONCE = {
    'backend': 'celery_once.backends.Redis',
    'settings': {
        'url': os.environ.get('QUEUE_URL', 'redis://localhost:6379/0'),
        'default_timeout': 3600,
    }
}
CELERY_TASK_ROUTES = {
    # Route all machine learning work to specific workers
    'arkindex_ml.*': {
        'queue': 'ml',
    }
}
CELERY_WORKING_DIR = os.environ.get('CELERY_WORKING_DIR', os.path.join(BASE_DIR, 'workers'))

# Email
EMAIL_SUBJECT_PREFIX = '[Arkindex {}] '.format(ARKINDEX_ENV)
if os.environ.get('EMAIL_HOST'):
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
    EMAIL_HOST = os.environ.get('EMAIL_HOST')
    EMAIL_PORT = os.environ.get('EMAIL_PORT')
    EMAIL_HOST_USER = os.environ.get('EMAIL_HOST_USER')
    DEFAULT_FROM_EMAIL = SERVER_EMAIL = EMAIL_HOST_USER
    EMAIL_HOST_PASSWORD = os.environ.get('EMAIL_HOST_PASSWORD')
    EMAIL_USE_TLS = True
else:
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Optional unit tests runner with code coverage
try:
    import django_nose # noqa
    INSTALLED_APPS.append('django_nose')
    TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
    NOSE_ARGS = [
        '--with-coverage',
        '--cover-package=arkindex',
    ]
except ImportError:
    pass

# Local settings
try:
    from .local_settings import * # noqa
except ImportError:
    pass


try:
    import debug_toolbar # noqa
    MIDDLEWARE.insert(0, 'debug_toolbar.middleware.DebugToolbarMiddleware')
    INSTALLED_APPS.append('debug_toolbar')
except ImportError:
    pass
