from rest_framework.generics import ListAPIView, RetrieveDestroyAPIView
from rest_framework.permissions import IsAuthenticated
from arkindex.users.providers import oauth_providers
from arkindex.users.serializers import OAuthCredentialsSerializer, OAuthProviderClassSerializer


class ProvidersList(ListAPIView):
    permission_classes = (IsAuthenticated, )
    serializer_class = OAuthProviderClassSerializer
    pagination_class = None

    def get_queryset(self):
        return list(filter(lambda p: p.enabled(), oauth_providers))


class CredentialsList(ListAPIView):
    permission_classes = (IsAuthenticated, )
    serializer_class = OAuthCredentialsSerializer

    def get_queryset(self):
        return self.request.user.credentials.exclude(token=None).order_by('id')


class CredentialsRetrieve(RetrieveDestroyAPIView):
    permission_classes = (IsAuthenticated, )
    serializer_class = OAuthCredentialsSerializer

    def get_queryset(self):
        return self.request.user.credentials.exclude(token=None).order_by('id')

    def perform_destroy(self, instance):
        instance.provider_class(request=self.request, credentials=instance).disconnect()
        super().perform_destroy(instance)
