from django.db import models
from django.contrib.auth.models import AbstractBaseUser
from arkindex.users.managers import UserManager
from arkindex.users.providers import oauth_providers, get_provider
import uuid


class User(AbstractBaseUser):
    email = models.EmailField(
        verbose_name='email address',
        max_length=255,
        unique=True,
    )
    is_active = models.BooleanField(default=True)
    is_admin = models.BooleanField(default=False)

    objects = UserManager()

    USERNAME_FIELD = 'email'

    def __str__(self):
        return self.email

    def has_perm(self, perm, obj=None):
        "Does the user have a specific permission?"
        # Simplest possible answer: Yes, always
        return True

    def has_module_perms(self, app_label):
        "Does the user have permissions to view the app `app_label`?"
        # Simplest possible answer: Yes, always
        return True

    @property
    def is_staff(self):
        "Is the user a member of staff?"
        # Simplest possible answer: All admins are staff
        return self.is_admin


class OAuthCredentials(models.Model):
    id = models.UUIDField(default=uuid.uuid4, primary_key=True)
    user = models.ForeignKey('users.User', on_delete=models.CASCADE, related_name='credentials')
    provider_name = models.CharField(
        max_length=50,
        choices=[(p.__name__, p.display_name) for p in oauth_providers],
    )
    provider_url = models.URLField()
    token = models.CharField(max_length=64, blank=True, null=True)
    refresh_token = models.CharField(max_length=64, blank=True, null=True)
    expiry = models.DateTimeField(blank=True, null=True)
    account_name = models.CharField(max_length=100, blank=True, null=True)

    @property
    def provider_class(self):
        return get_provider(self.provider_name)

    @property
    def provider(self):
        return self.provider_class(credentials=self)

    @property
    def git_provider_class(self):
        from arkindex.dataimport.providers import from_oauth
        return from_oauth(self.provider_name)

    @property
    def git_provider(self):
        return self.git_provider_class(credentials=self)
