from django.test import TestCase
from django.urls import reverse
from django.core import mail
from django.contrib import auth
from arkindex.users.models import User
from rest_framework.authtoken.models import Token


class TestUsers(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('email@address.com', 'P4$5w0Rd')

    def test_login_success(self):
        response = self.client.post(
            reverse('login'),
            data={'username': 'email@address.com', 'password': 'P4$5w0Rd'},
            follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(auth.get_user(self.client), self.user)

    def test_login_failure(self):
        response = self.client.post(
            reverse('login'),
            data={'username': 'email@address.com', 'password': 'nope'},
            follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertFalse(auth.get_user(self.client).is_authenticated)

    def test_logout(self):
        self.client.force_login(self.user)
        response = self.client.get(reverse('logout'), follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertFalse(auth.get_user(self.client).is_authenticated)

    def test_password_reset(self):
        response = self.client.post(
            reverse('password_reset'),
            data={'email': 'email@address.com'},
            follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertListEqual(response.redirect_chain, [(reverse('password_reset_done'), 302)])
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, ['email@address.com'])

    def test_auto_token(self):
        """
        Check creating a user automatically creates a token
        """
        self.assertEqual(Token.objects.filter(user=self.user).count(), 1)
