from django.views.generic import RedirectView, TemplateView
from django.contrib.auth.mixins import LoginRequiredMixin
from arkindex.users.providers import get_provider


class OAuthSignIn(LoginRequiredMixin, RedirectView):
    """
    Start a OAuth authentication workflow
    """

    def get_redirect_url(self, *args, **kwargs):
        if 'provider' not in kwargs:
            return
        provider = get_provider(kwargs['provider'])
        if not provider:
            return
        url = self.request.GET.get('url', provider.url)
        # Create OAuthCredentials without a token
        creds = self.request.user.credentials.create(
            provider_name=kwargs['provider'],
            provider_url=url,
        )
        return provider(request=self.request, credentials=creds).get_authorize_uri()


class OAuthCallback(LoginRequiredMixin, RedirectView):
    """
    Callback for OAuth responses
    """

    pattern_name = 'credentials'

    def get(self, request, *args, **kwargs):
        assert 'provider' in kwargs
        provider = get_provider(kwargs['provider'])
        if not provider:
            raise ValueError('Unknown provider')
        provider(self.request).handle_callback()
        return super().get(request)


class CredentialsList(LoginRequiredMixin, TemplateView):
    """
    View and manage OAuth providers
    """
    template_name = 'dataimport/credentials.html'
